/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "DesktopItem.h"
#include "DesktopTexture.h"

#include "shlwapi.h"

extern DesktopItemList desktopitemList;

extern UINT (WINAPI *_PrivateExtractIcons)(LPCTSTR,int,int,int,HICON*,UINT*,UINT,UINT);

DesktopTexture::DesktopTexture()
{
	useSmallIcon = false;

	noActiveIconColor = false;

	NormalGradient.clear();
	ActiveGradient.clear();

	linkIcon = NULL;
	activelinkIcon = NULL;
}

DesktopTexture::~DesktopTexture()
{
	NormalGradient.clear();
	ActiveGradient.clear();

	if ( linkIcon )
		DestroyIcon( linkIcon );
	if ( activelinkIcon )
		DestroyIcon( activelinkIcon );
}

void DesktopTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	int color = 0;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);
		
	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void DesktopTexture::parseGradientCFG(string cfg, int imageType)
{
	if ( cfg.empty() )
		return;

	int color = 0;
	COLORREF insertColor;
	char token[MAX_LINE_LENGTH] = {0};
	LPCSTR nextToken = cfg.c_str();
	
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == 0)
		{
			NormalGradient.insert(NormalGradient.end(), insertColor );
		}
		else if (imageType == 1)
		{
			ActiveGradient.insert(ActiveGradient.end(), insertColor );
		}
	}
}

void DesktopTexture::configure(const string &groupname)
{
	NameValuePair iconTextValues[] = {
		{ "top", 1 },
		{ "bottom", 2 },
		{ "left", 3 },
		{ "right", 4 },
		{ 0, 0 }
	};

	useNoIcon = GetxRCBool(groupname, "NoIcon");
	iconSize = GetxRCInt(groupname, "IconSize", 32, 16, 64);

	noActiveIconColor = GetxRCBool(groupname, "NoActiveIconColor");
	
	normal_saturation = GetxRCInt(groupname, "SaturationIntensity", 255, 0, 255);
	normal_hueIntensity = GetxRCInt(groupname, "HueIntensity", 0, 0, 255);
	normal_hueColor = GetxRCColor(groupname, "HueColor", RGB(0, 0, 0));

	active_saturation = GetxRCInt(groupname, "ActiveSaturationIntensity", normal_saturation, 0, 255);
	active_hueIntensity = GetxRCInt(groupname, "ActiveHueIntensity", normal_hueIntensity, 0, 255);
	active_hueColor = GetxRCColor(groupname, "ActiveHueColor", normal_hueColor);

	linkOverlayIcon = GetxRCString(groupname, "LinkOverlayIcon", "shell32.dll,29");
	// Get the icon index
	int nIconIndex = 0;
	int nIndexStart = linkOverlayIcon.rfind(',');
	if(nIndexStart != std::string::npos)
	{
		nIconIndex = atoi(linkOverlayIcon.substr(nIndexStart + 1).c_str());
		linkOverlayIcon = linkOverlayIcon.substr(0, nIndexStart);
	}
	
	HICON tmpIcon = NULL;
	HICON tmpactiveIcon = NULL;

	if ( _PrivateExtractIcons != NULL )
		_PrivateExtractIcons(linkOverlayIcon.c_str(), nIconIndex, iconSize, iconSize, &tmpIcon, NULL, 1, NULL);

	if ( tmpIcon )
	{
		tmpactiveIcon = CopyIcon(tmpIcon);
		if (normal_saturation != 255 || normal_hueIntensity != 0 )
			linkIcon = applyIconEffects(tmpIcon, false);
		else
			linkIcon = tmpIcon;
		//Apply SystemColor
		activelinkIcon = applyIconEffects(tmpactiveIcon, true, true);	
		//Apply User Defined Color
		if ( active_saturation != 255 || active_hueIntensity != 0 )
			activelinkIcon = applyIconEffects(activelinkIcon, true);
	}

	iconTextPosition = GetxRCNamedValue(groupname, "TextPosition", iconTextValues, 2);
	iconTextSpacing = GetxRCInt(groupname, "TextSpacing", 5);

	useSmallIcon = GetxRCBool(groupname, "UseSmallIcon");
	
	string normaltextBG = GetxRCLine(groupname, "TextBackground", (useNoIcon ? "c0c0c0 555555" : "ff00ff"));
	parseCFG(normaltextBG, iconTextBackground, iconTextBackgroundLight, iconTextBackgroundDark);
	//BG Insert in Gradient List
	NormalGradient.insert(NormalGradient.end(), iconTextBackground );
	
	parseCFG(GetxRCLine(groupname, "ActiveTextBackground", normaltextBG), activeiconTextBackground, activeiconTextBackgroundLight, activeiconTextBackgroundDark);
	//BG Insert in Gradient List
	ActiveGradient.insert(ActiveGradient.end(), activeiconTextBackground );

	string normalgradientColor = GetxRCLine(groupname, "TextBackgroundGradientColors", "");
	parseGradientCFG(normalgradientColor, 0);	
	normalverticalGradient = GetxRCBool(groupname, "TextBackgroundGradientVertical");
	normalrepeatedGradient = GetxRCInt(groupname, "TextBackgroundGradientRepeated", 0, 0);

	parseGradientCFG(GetxRCLine(groupname, "ActiveTextBackgroundGradientColors", normalgradientColor), 1);
	activeverticalGradient = GetxRCBool(groupname, "ActiveTextBackgroundGradientVertical", normalverticalGradient);
	activerepeatedGradient = GetxRCInt(groupname, "ActiveTextBackgroundGradientRepeated", normalrepeatedGradient, 0);
}

void DesktopTexture::apply(HDC hDC, int x, int y, int width, int height, POINT size, const string &iconCommand, const string &uniquekey, bool activated)
{
	HICON hIcon = NULL;

	if ( !useNoIcon )
	{
		if ( StrStrI(iconCommand.c_str(), ".lnk") != NULL )
		{
			// Special case for link files
			// If the icon is set for the link, retrieve it
			SHFILEINFO shfi;
			SHGetFileInfo(iconCommand.c_str(), 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICONLOCATION);
			// Extract the icon
			if ( _PrivateExtractIcons != NULL )
				_PrivateExtractIcons(shfi.szDisplayName, shfi.iIcon, iconSize, iconSize, &hIcon, NULL, 1, NULL);

			// If the icon was not explicitly set in the linkfile, we'll use the icon from the target file
			if(hIcon == NULL) 
			{
				HRESULT hRes;				
				char szInfo[1024];
				IShellLink* psl;

				// Create the ShellLink object
				hRes = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID*)&psl);
				if(SUCCEEDED(hRes))
				{
					IPersistFile* ppf;
					// Bind the ShellLink object to the Persistent File
					hRes = psl->QueryInterface(IID_IPersistFile, (LPVOID *) &ppf);
					if(SUCCEEDED(hRes))
					{
						WORD wsz[MAX_PATH];

						MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, iconCommand.c_str(), -1, wsz, MAX_PATH);
						hRes = ppf->Load(wsz, 0);
							
						if(SUCCEEDED(hRes))
						{
							// Retrieve the target of the link file
							psl->GetPath(szInfo, 1024, NULL, SLGP_UNCPRIORITY);
						}
					}
				}
				psl->Release();

				// Extract the icon from the link target
				SHFILEINFO shfi;
				SHGetFileInfo((LPCSTR) szInfo, 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICON | ((activated && !noActiveIconColor) ? SHGFI_SELECTED : 0) | (useSmallIcon ? SHGFI_SMALLICON : SHGFI_LARGEICON) );
				hIcon = shfi.hIcon;
			}
		}
		else
		{
			SHFILEINFO shfi;
			SHGetFileInfo((LPCSTR) iconCommand.c_str(), 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICON | ((activated && !noActiveIconColor) ? SHGFI_SELECTED : 0) | (useSmallIcon ? SHGFI_SMALLICON : SHGFI_LARGEICON) );
			hIcon = shfi.hIcon;
		}
		
		if(hIcon == NULL)
			return;
	}
			
	HDC	memoryDC = CreateCompatibleDC(hDC);
	HBITMAP memoryBitmap = CreateCompatibleBitmap(hDC, width, height);
	HBITMAP oldmemoryBMP = (HBITMAP) SelectObject(memoryDC, memoryBitmap);

	RECT r_Icon;
	r_Icon.top = 0;
	r_Icon.left = 0;
	r_Icon.right = width;
	r_Icon.bottom = height;

	::SetBkColor(memoryDC, RGB(255, 0, 255));
	::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r_Icon, NULL, 0, NULL);

	int xpos = 0;
	int ypos = 0;

	if ( useNoIcon )
	{
		iconSize = 0;
		iconTextSpacing = 0;
	}

	if (iconTextPosition == 1)
	{
		xpos = (width-iconSize)/2;
		ypos = height-iconSize;
	
		r_Icon.left = max((width-size.x)/2, 0);
		r_Icon.right = min(r_Icon.left+size.x, width);
		r_Icon.top = max(height-(iconSize+iconTextSpacing+size.y), 0);
		r_Icon.bottom = min(r_Icon.top+size.y, height);
	}
	else if (iconTextPosition == 2)
	{
		xpos = (width-iconSize)/2;
		ypos = 0;
	
		r_Icon.left = max((width-size.x)/2, 0);
		r_Icon.right = min(r_Icon.left+size.x, width);
		r_Icon.top = iconSize+iconTextSpacing;
		r_Icon.bottom = min(r_Icon.top+size.y, height);
	}
	else if (iconTextPosition == 3)
	{
		xpos = width-iconSize;
		ypos = (height-iconSize)/2;
	
		r_Icon.left = max(width-iconSize-iconTextSpacing-size.x, 0);
		r_Icon.right = min(r_Icon.left+size.x, width-iconSize-iconTextSpacing);
		r_Icon.top = max((height-size.y)/2, 0);
		r_Icon.bottom = min(r_Icon.top+size.y, height);
	}
	else if (iconTextPosition == 4)
	{
		xpos = 0;
		ypos = (height-iconSize)/2;
		
		r_Icon.left = iconSize+iconTextSpacing;
		r_Icon.right = min(r_Icon.left+size.x, width);
		r_Icon.top = max((height-size.y)/2, 0);
		r_Icon.bottom = min(r_Icon.top+size.y, height);
	}

	if ( activated )
	{
		::SetBkColor(memoryDC, activeiconTextBackgroundDark);
		::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r_Icon, NULL, 0, NULL);

		r_Icon.right = r_Icon.right-1;
		r_Icon.bottom = r_Icon.bottom-1;
		::SetBkColor(memoryDC, activeiconTextBackgroundLight);
		::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r_Icon, NULL, 0, NULL);

		r_Icon.left = r_Icon.left+1;
		r_Icon.top = r_Icon.top+1;
		if ( ActiveGradient.size() > 1 )
		{
			solidGradientPaint(memoryDC, ActiveGradient, r_Icon, activeverticalGradient, activerepeatedGradient);
		}
		else
		{
			::SetBkColor(memoryDC, activeiconTextBackground);
			::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r_Icon, NULL, 0, NULL);
		}
	}
	else
	{
		::SetBkColor(memoryDC, iconTextBackgroundDark);
		::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r_Icon, NULL, 0, NULL);

		r_Icon.right = r_Icon.right-1;
		r_Icon.bottom = r_Icon.bottom-1;
		::SetBkColor(memoryDC, iconTextBackgroundLight);
		::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r_Icon, NULL, 0, NULL);

		r_Icon.left = r_Icon.left+1;
		r_Icon.top = r_Icon.top+1;
		if ( NormalGradient.size() > 1 )
		{
			solidGradientPaint(memoryDC, NormalGradient, r_Icon, normalverticalGradient, normalrepeatedGradient);
		}
		else
		{
			::SetBkColor(memoryDC, iconTextBackground);
			::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r_Icon, NULL, 0, NULL);
		}
	}

	if ( !useNoIcon )
	{
		if ( IsOS(OS_XP) || (!activated && (normal_saturation != 255 || normal_hueIntensity != 0)) || (activated && (active_saturation != 255 || active_hueIntensity != 0)) )
		{	
			if ( IsOS(OS_XP) )
			{	
				int curx = 0;
				int cury = 0;
				for ( DesktopItemListIterator iter = desktopitemList.begin(); iter != desktopitemList.end(); iter++)
				{
					if(stricmp((*iter)->getUniqueKey().c_str(), uniquekey.c_str()) == NULL)
					{
						curx = (*iter)->getX();
						cury = (*iter)->getY();
						break;
					}
				}

				PaintDesktopEx(memoryDC, 0, 0, iconSize, iconSize, curx+xpos, cury+ypos, false);
			}

			hIcon = applyIconEffects(hIcon, activated);

			DrawIconEx(memoryDC, xpos, ypos, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);
			if ( StrStrI(iconCommand.c_str(), ".lnk") != NULL )
				DrawIconEx(memoryDC, xpos, ypos, (activated ? activelinkIcon : linkIcon), iconSize, iconSize, 0, NULL, DI_NORMAL);
		}
		else
		{
			DrawIconEx(memoryDC, xpos, ypos, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);
			if ( StrStrI(iconCommand.c_str(), ".lnk") != NULL )
				DrawIconEx(memoryDC, xpos, ypos, (activated ? activelinkIcon : linkIcon), iconSize, iconSize, 0, NULL, DI_NORMAL);
		}
	}

	BitBlt( hDC, x, y, width, height, memoryDC, 0, 0, SRCCOPY );

	memoryBitmap = (HBITMAP) SelectObject(memoryDC, oldmemoryBMP);
	DeleteObject(memoryBitmap);
	DeleteDC(memoryDC);

	if ( hIcon )
		DestroyIcon(hIcon);
}

void DesktopTexture::solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, bool vertical, int repeated)
{
	if ( repeated == 0 )
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		double cr, cg, cb, dr, dg, db;

		// ColorsSteps between Start and End Color
		int gradSteps = 0;
		int unequalRest = 0;

		int colorCount = workGradient.size();

		if (vertical)
		{
			gradSteps = (nHeight-colorCount)/(colorCount-1);
			unequalRest = (nHeight-colorCount)%(colorCount-1);
		}
		else
		{
			gradSteps = (nWidth-colorCount)/(colorCount-1);
			unequalRest = (nWidth-colorCount)%(colorCount-1);
		}

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
		{
			if (cit == workGradient.end())
				break;

			counter++;

			COLORREF one = *cit;
			cit++;
			COLORREF two = *cit;
			cit--;
		
			// get current color
			cr = GetRValue( one );
			cg = GetGValue( one );
			cb = GetBValue( one );

			// Get delta color (+1 cause we must reach the Final Color)
			dr = (cr - GetRValue( two )) / (gradSteps+1);
			dg = (cg - GetGValue( two )) / (gradSteps+1);
			db = (cb - GetBValue( two )) / (gradSteps+1);

			//Paint Real Gradient Color
			//-------------------------------------------
			if ( unequalRest != 0 && counter > 1 )
			{
				unequalRest--;
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, one);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;
			}
			//-------------------------------------------

			for (int i = 0; i <= gradSteps; i++)
			{
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos+i, r.right, min(lastGradientPos+i+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos+i, r.top, min(lastGradientPos+i+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				cr -= dr;
				cg -= dg;
				cb -= db;
			}

			if (vertical)
				lastGradientPos = r2.bottom;
			else
				lastGradientPos = r2.right;
		}
	}
	else
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		COLORREF color;

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		int nSize = 0;
		
		if (vertical)
			nSize = nHeight;
		else
			nSize = nWidth; 

		while ( counter < nSize )
		{
			for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
			{
				color = *cit;

				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+repeated, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+repeated, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0 && color != RGB(255,0,255) )
				{
					::SetBkColor(destHDC, color);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}

				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;

				counter++;

				if ( counter == nSize )
					break;
			}
		}
	}
}

HICON DesktopTexture::applyIconEffects(HICON hIcon, bool active, bool systemactive)
{
	if(hIcon == NULL)
		return NULL;

	HICON resulthIcon = NULL;

	int _saturation = normal_saturation;
	int _hueIntensity = normal_hueIntensity;
	COLORREF _hueColor = normal_hueColor;
	if ( active )
	{
		_saturation = active_saturation;
		_hueIntensity = active_hueIntensity;
		_hueColor = active_hueColor;
	}

	if ( systemactive )
	{
		_saturation = 255;
		_hueIntensity = 135;
		_hueColor = GetSysColor(COLOR_HIGHLIGHT);
	}

	ICONINFO ii;
	GetIconInfo(hIcon, &ii);

	HDC bufferDC = CreateCompatibleDC(NULL);
	HBITMAP oldbufferBMP = (HBITMAP) SelectObject(bufferDC, ii.hbmColor);

	HDC maskDC = CreateCompatibleDC(NULL);
	HBITMAP maskBitmap = CreateCompatibleBitmap(maskDC, iconSize, iconSize);
	HBITMAP oldmaskBMP = (HBITMAP) SelectObject(maskDC, maskBitmap);

	// Mask of the icon
	DrawIconEx(maskDC, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_MASK);

	int y = 0;
	int x = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;
	for(y = 0; y < iconSize; y++)
	{
		for(x = 0; x < iconSize; x++)
		{
			// Loop through all transparent pixels...
			// If the mask-pixel is white, then break
			if(!GetPixel(maskDC, x, y))
			{
				cl = GetPixel(bufferDC, x, y);
				r = GetRValue(cl);
				g = GetGValue(cl);
				b = GetBValue(cl);

				// Saturation effect
				if(_saturation != 255)
				{
					BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

					r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
					g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
					b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
				}

				// Hue effect
				if(_hueIntensity != 0)
				{
					// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
					// test normal
					r = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
				}

				SetPixelV(bufferDC, x, y, RGB(r, g, b));
			}
		}
	}

	maskBitmap = (HBITMAP) SelectObject(maskDC, oldmaskBMP);
	DeleteObject(maskBitmap);
	DeleteDC(maskDC);

	ii.hbmColor = (HBITMAP) SelectObject(bufferDC, oldbufferBMP);
	
	DestroyIcon(hIcon);

	resulthIcon = CreateIconIndirect(&ii);
	
	DeleteObject(ii.hbmColor);
	DeleteObject(ii.hbmMask);
	DeleteDC(bufferDC);

	return resulthIcon;
}